import requests
import xml.etree.ElementTree as ET

# TODO: We need to handle case where no game has started yet,
#       i.e. the url returns a 404 not found error. Currently
#       this returns a ET.ParseError


class MlbParser:
    
    def __init__(self, url):
        self.url = url
        self.last_play = None
        self.score = None

    def get_xml(self):
        return requests.get(self.url).content

    @staticmethod
    def get_score(play_tree):
        try:
            score_tree = play_tree.findall('dict')[-1]
            scores_list = score_tree.findall('integer')
            return {'away': int(scores_list[0].text), 'home': int(scores_list[1].text)}
        except AttributeError:
            return {'away': 0, 'home': 0}

    def get_play(self, xml_string):
        try:
            root = ET.fromstring(xml_string)
        except ET.ParseError:
            # TODO: Log parse error here
            return None
        array = root.find('dict').find('array')
        if len(array.findall('dict')) > 0:
            new_play = array.findall('dict')[-1]
        else:
            return None
        timestamp = new_play.findall('date')[-1]
        if self.last_play is not None:
            if timestamp != self.last_play.findall('date')[-1]:
                self.last_play = new_play
        else:
            self.last_play = new_play
        return self.last_play



import sys
import logging
from time_converter import TimeConverter
from parse import MlbParser
from url_builder import UrlBuilder
from time_converter import TimeConverter
from database import PickleDatabase
from client import Client


def initialize_main_logger():
    logging.basicConfig(level=logging.INFO)
    handler = logging.FileHandler('mlb.log')
    handler.setLevel(logging.INFO)
    logger = logging.getLogger(__name__)
    logger.addHandler(handler)
    return logger

def initialize_error_logger():
    logging.basicConfig(level=logging.ERROR)
    handler = logging.FileHandler('error.log')
    handler.setLevel(logging.ERROR)
    logger = logging.getLogger(__name__)
    logger.addHandler(handler)
    return logger

def main():
    logger = initialize_main_logger()
    error_logger = initialize_error_logger()
    converted_time = TimeConverter().get_time()
    db = PickleDatabase()
    db.seed()
    team_name = ' '.join(sys.argv[1:])
    if len(team_name) > 0:
        team_code = db.get_team_code(team_name)
        if team_code is None:
            error_logger.error(TimeConverter().get_timestamp() + "\tTeam %s not found", team_name)
            return 1
        else:
            logger.info("Team code: %s", team_code)
            url = UrlBuilder(converted_time, team_code).build_url()
            mlb = MlbParser(url)
            c = Client(mlb)
            c.update()
    else:
        error_logger.error(TimeConverter().get_timestamp() + "\tNo team specified")
        print "Please specify a team."
        return 1

if __name__ == "__main__":
    main()



__author__ = 'ecowan'

import requests
import re
from bs4 import BeautifulSoup


class LinkFinder:

    def __init__(self, url, team_name=None):
        self.url = url
        self.team_name = team_name

    def get_link(self):
        text = requests.get(self.url).content
        soup = BeautifulSoup(text)
        if self.team_name is None:
            # TODO: Log an error here before going to default case
            self.team_name = "wasmlb"
        else:
            self.team_name = self.team_name+"mlb"
        filtered_links = soup.find_all(href=re.compile(self.team_name))
        if len(filtered_links) == 1:
            return str(filtered_links[0]['href'])
        else:
            return None


import unittest
from parse import MlbParser
from time_converter import TimeConverter
from url_builder import UrlBuilder
from link_finder import LinkFinder
from database import PickleDatabase

# TODO: Pass time structs very early in morning before games to test ParseError catching (see todo in parse.py)

class ParserTest(unittest.TestCase):

    def setUp(self):
        self.mlb = MlbParser("http://gd2.mlb.com/components/game/mlb/year_2015/month_04/day_21/gid_2015_04_21_slnmlb_wasmlb_1/runScoringPlays.plist")
        self.xml = open('data/test.xml', 'r').read()
        self.play = self.mlb.get_play(self.xml)
        self.score = self.mlb.get_score(self.play)

    def testPlayIsNotNone(self):
        self.assertIsNotNone(self.play)

    def testScoreIsNotNone(self):
        self.assertIsNotNone(self.score)

    def testAwayHasOneRun(self):
        self.assertEqual(self.score['away'], 1)

    def testHomeHasTwoRuns(self):
        self.assertEqual(self.score['home'], 2)


class TimeConverterTest(unittest.TestCase):

    def setUp(self):
        self.converter = TimeConverter()

    def testYear(self):
        self.eastern_time = self.converter.get_time()
        self.assertGreaterEqual(self.eastern_time['year'], 2015)


class UrlBuilderTest(unittest.TestCase):

    def setUp(self):
        self.time_dict = TimeConverter().get_time()
        self.builder = UrlBuilder(self.time_dict, "was")

    def testPadSingleDigit(self):
        self.assertEqual('04', self.builder.pad_single_digit(4))

    def testPadDoubleDigit(self):
        self.assertEqual('10', self.builder.pad_single_digit(10))

    def testSearchUrl(self):
        self.assertIsNotNone(self.builder.build_search_url())

    def testWholeUrl(self):
        self.assertIsNotNone(self.builder.build_url())


class LinkFinderTest(unittest.TestCase):

    def setUp(self):
        self.search_url = "http://gd2.mlb.com/components/game/mlb/year_2015/month_04/day_22/"
        self.link_finder = LinkFinder(self.search_url)

    def testUrlIsFound(self):
        self.assertIsNotNone(self.link_finder.get_link())

class TeamCodeLookupTest(unittest.TestCase):

    def setUp(self):
        self.db = PickleDatabase()
        self.db.seed()

    def testCorrectKey(self):
        self.assertEqual('bal', self.db.get_team_code('Baltimore Orioles'))

    def testMisspelledKey(self):
        self.assertEqual('bal', self.db.get_team_code('Blatimore Orioles'))

    def testDodgersIncomplete(self):
        self.assertEqual('lad', self.db.get_team_code('LA Dodgers'))

    def testAngelsIncomplete(self):
        self.assertEqual('laa', self.db.get_team_code('LA Angels'))

    def testLAIncomplete(self):
        self.assertGreaterEqual(len(self.db.get_team_code('LA')), 2)



__author__ = 'ecowan'

import threading
import logging
from time_converter import TimeConverter


class Client:

    def __init__(self, mlb_parser, update_interval=1.25):
        self.mlb_parser = mlb_parser
        self.update_interval = update_interval
        self.logger = None
        self.initialize_logger()

    def initialize_logger(self):
        logging.basicConfig(level=logging.INFO)
        handler = logging.FileHandler('mlb.log')
        handler.setLevel(logging.INFO)
        self.logger = logging.getLogger(__name__)
        self.logger.addHandler(handler)
        self.logger.info(TimeConverter().get_timestamp() + '\tStarting Realtime Baseball Updates')

    def update(self):
        threading.Timer(self.update_interval, self.update).start()
        xml = self.mlb_parser.get_xml()
        play = self.mlb_parser.get_play(xml)
        if play is not None:
            if self.mlb_parser.score is not None:
                new_score = self.mlb_parser.get_score(self.mlb_parser.last_play)
                if new_score != self.mlb_parser.score:
                    self.mlb_parser.score = new_score
                    self.logger.info("%s\t%s", self.mlb_parser.last_play.findall('string')[-1].text, self.mlb_parser.score['home'])
            else:
                self.mlb_parser.score = self.mlb_parser.get_score(self.mlb_parser.last_play)
                if self.mlb_parser.last_play is not None:
                    self.logger.info("%s\thome:\t%s\taway:\t%s", self.mlb_parser.last_play.findall('string')[-1].text, self.mlb_parser.score['home'], self.mlb_parser.score['away'])





__author__ = 'ecowan'

from datetime import datetime, timedelta
from pytz import timezone
import pytz

class TimeConverter:

    def __init__(self):
        self.utc = pytz.utc
        self.eastern = timezone('US/Eastern')

    def get_timestamp(self):
        return str(datetime.now())

    def get_time(self):
        t = datetime.now()
        eastern_time = self.eastern.localize(t)
        return {'year': eastern_time.year,
                'month': eastern_time.month,
                'day': eastern_time.day}


__author__ = 'ecowan'


from time_converter import TimeConverter
from link_finder import LinkFinder

# TODO: Need to handle case where team is not playing that day.
# Currently returns: 
'''
Traceback (most recent call last):
  File "/usr/local/bin/mlb-realtime", line 9, in <module>
    load_entry_point('mlb-realtime==0.0.13', 'console_scripts', 'mlb-realtime')()
  File "build/bdist.linux-x86_64/egg/realtime/__main__.py", line 41, in main
  File "build/bdist.linux-x86_64/egg/realtime/url_builder.py", line 29, in build_url
TypeError: sequence item 1: expected string, NoneType found
'''


class UrlBuilder:

    def __init__(self, time_dict, team_code):
        self.base_url = "http://gd2.mlb.com/components/game/mlb/"
        self.time_dict = dict((k, self.pad_single_digit(v)) for k,v in time_dict.items())
        self.team_code = team_code

    def pad_single_digit(self, num):
        if num < 10:
            return '0' + str(num)
        else:
            return str(num)

    def build_search_url(self):
        return self.base_url +  "/".join(["year_"+self.time_dict['year'], "month_"+self.time_dict['month'], "day_"+self.time_dict['day']])

    def get_gid(self):
        return LinkFinder(self.build_search_url(), self.team_code).get_link()

    def build_url(self):
        search_url = self.build_search_url()
        gid = self.get_gid()
        return "/".join([search_url, gid]) + "runScoringPlays.plist"



__author__ = 'ecowan'

import time
import logging
import pickledb


def db_logger():
    logger = logging.getLogger(__name__)
    handler = logging.FileHandler('error.log')
    handler.setLevel(logging.ERROR)
    logger.addHandler(handler)

teams_string = '''Arizona Diamondbacks	ARI
Atlanta Braves	ATL
Baltimore Orioles	BAL
Boston Red Sox	BOS
Chicago Cubs	CHN
Chicago White Sox	CHA
Cincinnati Reds	CIN
Cleveland Indians	CLE
Colorado Rockies	COL
Detroit Tigers	DET
Florida Marlins	FLA
Houston Astros	HOU
Kansas City Royals	KAN
Los Angeles Angels of Anaheim	LAA
Los Angeles Dodgers	LAD
Milwaukee Brewers	MIL
Minnesota Twins	MIN
New York Mets	NYM
New York Yankees	NYY
Oakland Athletics	OAK
Philadelphia Phillies	PHI
Pittsburgh Pirates	PIT
San Diego Padres	SD
San Francisco Giants	SF
Seattle Mariners	SEA
St. Louis Cardinals	STL
Tampa Bay Rays	TB
Texas Rangers	TEX
Toronto Blue Jays	TOR
Washington Nationals	WAS'''

class PickleDatabase:

    def __init__(self, logger=db_logger()):
        self.db = pickledb.load('mlb.db', False)
        self.logger = logger or logging.getLogger(__name__)

    def seed(self):
        for line in teams_string.split('\n'):
            line_list = line.split('\t')
            team_name = line_list[0].lower().strip()
            team_code = line_list[1].lower().strip()
            self.db.set(team_name, team_code)

    @staticmethod
    def get_city(team_name):
        if len(team_name.split(' ')) > 1:
            return team_name.split(' ')[-1]
        else:
            return team_name

    def get_team_code(self, team_name):
        team_name = team_name.lower()
        team_code = self.db.get(team_name)
        if team_code is None:
            city = self.get_city(team_name)
            team_code_key = [x for x in self.db.db.keys() if city in x]
            if len(team_code_key) == 1:
                return self.db.get(team_code_key[0])
            elif 'dod' in team_name: #LA team names making life difficult
                return 'lad'
            elif 'ana' in team_name:
                return 'laa'
            elif len(team_code_key) > 1:
                self.logger.error("%s\nSearched for:\t%s", time.ctime(), team_name)
                self.logger.error("More than one team found:\t%s\n", str(team_code_key))
                return team_code_key
            elif len(team_code_key) == 0:
                self.logger.error("%s\nSearched for:\t%s", time.ctime(), team_name)
                self.logger.error("No team could be found:\n")
                return None
        else:
            return team_code